
/* Hive Spin Compiler CON block parsing */
/* This code is public domain */

#include <stdlib.h>
#include "keyword.h"
#include "symbol.h"
#include "parse.h"
#include "hsc.h"

/* Parse a CON block and add the constants to the symbol table.
 * Initializers with forward references are left to the 2nd pass
 * for fixing up.
 */
void ParseCONpass1()
{
  int idx = 0, inc;
  Symbol *sym;
  
  MustSkip(TK_CON);
  TestAndSkip(TK_EOL);
  while(1) {
    /* Test for the #n syntax (i.e. set enum index) */
    if( TestAndSkip(TK_HASH) ) {
      idx = EvalExpr( NULL );
    }
    /* else it must be an identifier; if not, we're done with the CON block */
    else {
      if( token.type!=TK_ID ) break;
      
      sym = AddSymbol(token.text, &(curobj->Globals));
      sym->type = INT_CON_SYMBOL;
      NextToken();
      
      /* Test for 'id = n' syntax */
      if( TestAndSkip(TK_EQUAL) ) {
        sym->value = TryEvalExpr( NULL );
        if( !evalOK ) sym->type = UNDEF_CON_SYMBOL;
      }
      /* Test for 'id[n]' syntax */
      else if( TestAndSkip(TK_LBRACKET) ) {
        inc = EvalExpr( NULL );
        MustSkip(TK_RBRACKET);
        sym->value = idx;
        idx += inc;
      }
      /* Handle the plain 'id' case */
      else {
        sym->value = idx;
        ++idx;
      }
    }
    /* Move to next constant */
    if( token.type==TK_COMMA || token.type==TK_EOL ) {
      NextToken();
    }
    else {
      hsc_abort("syntax error in constant declaration");
    }
  }
}

/* Pass 2 is simple: it just eats its way through the syntax and 
 * tidies up any constants that were left undefined on the first
 * pass.
 */
void ParseCONpass2()
{
  int dump;
  Symbol *sym;
    
  MustSkip(TK_CON);
  TestAndSkip(TK_EOL);
  while(1) {
    if( TestAndSkip(TK_HASH) ) {
      dump = EvalExpr( 0 );
    }
    else {
      if( token.type!=TK_ID ) break;
      
      sym = SymbolLookup(token.text, curobj->Globals);
      NextToken();
      
      /* here we do the actual work: fix up constants that might
       * have been left undefined in the first pass.
       */
      if( TestAndSkip(TK_EQUAL) ) {
        sym->value = EvalExpr( 0 );
        sym->type = INT_CON_SYMBOL;
      }
      else if( TestAndSkip(TK_LBRACKET) ) {
        dump = EvalExpr( 0 );
        MustSkip(TK_RBRACKET);
      }
    }
    if( token.type==TK_COMMA || token.type==TK_EOL ) {
      NextToken();
    }
    else {
      hsc_abort("internal error: syntax handling in pass 2");
    }
  }
}
