
/* Hive Spin Compiler symbol table */
/* This code is public domain */

#include "lexer.h"

/* symbol types; note: these must not overlap with TK_ codes */
#define INT_CON_SYMBOL   0xf0
#define FLT_CON_SYMBOL   0xf1
#define UNDEF_CON_SYMBOL 0xf2
#define OBJ_SYMBOL       0xf3
#define DAT_SYMBOL       0xf4
#define VAR_SYMBOL       0xf5
#define LCL_SYMBOL       0xf6
#define PUB_SYMBOL       0xf7
#define PRI_SYMBOL       0xf8

#define GetVarArea(t) (t-DAT_SYMBOL)

/* symbols are stored as a binary tree */
struct _symbol {
  struct _symbol *left;            /* left subtree */
  struct _symbol *right;           /* right subtree */
  union {                          /* symbol value (constants) or address offset (variables) */
    int value;
    float fltval;
    int offset;
    struct _object *obj;
  };
  unsigned char type;              /* symbol type: see above defines for list */
  char size;                       /* var symbol: size in bytes 1 (byte), 2 (word), 4 (long) */
  char idx;                        /* pub/pri symbol: index of method */
  char argc;                       /* pub/pri symbol: parameter count */
  char name[MAXTOKENLENGTH+1];     /* name of symbol (identifier) encoded as latin-1 string */
};
typedef struct _symbol Symbol;

/* symbol node extension for DAT symbols */
struct _datsym {
  Symbol *next_lcl;                /* linked list of local labels following this global label */
  int hubloc;                      /* offset of label in hubram */
  int cogloc;                      /* offset of label in cogram */
};
typedef struct _datsym DATSym;

/* symbol node extension for OBJ symbols */
struct _objsym {
  Symbol *sym_table;               /* symbol table of imported object */
  int  idx;                        /* index of object */
  int  count;                      /* no. of objects imported (OBJ name[count] : "file" syntax) */
  Symbol *next_obj;                /* linked list of objects */
};
typedef struct _objsym OBJSym;

/* the symbol table API */
extern Symbol *Locals;
Symbol *SymInit();
Symbol *SymbolLookup(char *str, Symbol *st);
Symbol *AddSymbol(char *str, Symbol **st);
Symbol *AllocSymbol(char *name);
DATSym *MakeDATSym(Symbol *sym);
void LocalsInit();
void LocalsFree();


void PrintSymbols(Symbol *st);
