/* This code is public domain */

#include <stdio.h>
#include <stdlib.h>

#include "keyword.h"
#include "symbol.h"
#include "parse.h"
#include "hsc.h"

int work(char *path);

int main(int argc, char **argv)
{  
  if( argc<2 ) {
    printf("usage: hsc <file>\n");
    exit(1);
  }
/*  printf("-----\n");
  hubptr = 0;
  ParsePass1(argv[1]);
  PrintSymbols( Global );
  printf("hubptr = %x\n", hubptr);
  printf("-----\n");
  hubptr = 0;
  ParsePass2(argv[1]);
  printf("hubptr = %x\n", hubptr);
  printf("-----\n");
*/
  work( argv[1] );
  return 0;
}

int pass;

int work(char *path)
{
  Object *obj;
  
  pass = 1;
  topobj = curobj = NewObject(path);
  
  EmitProgramHdr();
  ParsePass1(path);
  EmitProgramTail();
  
  pass = 2;
  EmitProgramHdr();
  for(obj = topobj; obj; obj = obj->next) {
    curobj = obj;
    ParsePass2(obj->path);
  }
  EmitProgramTail();
  
  SaveBinary("spin.binary");
  return 0;
}

void hsc_abort(char *str)
{
  printf("\n\nError in line %d, column %d: %s\n\n", token.line, token.col, str);
  exit(1);
}

void SkipBlock()
{
  NextToken();
  while( !IsBlockDesignator(token.type) && token.type!=TK_EOF ) NextToken();
}

void ParsePass1(char *path)
{
  EmitObjectHdr(curobj);
  OpenFile(path);
  while( token.type!=TK_EOF ) {
    int t = token.type;
    if( !IsBlockDesignator(t) ) hsc_abort("block designator expected");
    if( token.col ) hsc_abort("block designator not at start of line");
    
    switch( t ) {
      case TK_CON: ParseCONpass1(); break;
      case TK_VAR: ParseVARpass1(); break;
      case TK_DAT: ParseDATpass1(); break;
      case TK_OBJ: ParseOBJpass1(); break;
      case TK_PUB:
      case TK_PRI: ParsePUBPRIpass1(); break;
    }
  }
  CloseFile();
  EmitObjectTail(curobj);
}

void ParsePass2(char *path)
{
  EmitObjectHdr(curobj);
  OpenFile(path);
  while( token.type!=TK_EOF ) {
    int t = token.type;
    /* syntax has already been checked */
    
    switch( t ) {
      case TK_CON: ParseCONpass2(); break;
      case TK_VAR: ParseVARpass2(); break;
      case TK_DAT: ParseDATpass2(); break;
      case TK_OBJ: ParseOBJpass2(); break;
      case TK_PUB:
      case TK_PRI: ParsePUBPRIpass2(); break;
    }
  }
  CloseFile();
  EmitObjectTail(curobj);
}

