
/* Hive Spin Compiler VAR block parsing */
/* This code is public domain */

#include <stdlib.h>
#include "keyword.h"
#include "symbol.h"
#include "parse.h"
#include "hsc.h"

/* variables are allocated in three pools, one for each size
 * specifier (long, word, byte). curobj->varOffsets[x] holds the first
 * free offset location for each pool.
 */

void ParseVARpass1()
{
  Symbol *sym;
  int s, dim;
  
  MustSkip(TK_VAR);
  TestAndSkip(TK_EOL);
  while( IsSize(token.type) ) {
    switch( token.type ) {
      case TK_LONG: s = 2; break;
      case TK_WORD: s = 1; break;
      case TK_BYTE: s = 0;
    }
    NextToken();
    do {
      /* first read the variable name and create the symbol */
      if( !IsId(token.type) ) hsc_abort("expected identifier");
      sym = AddSymbol(token.text, &(curobj->Globals));
      sym->type = VAR_SYMBOL;
      sym->size = (1<<s);

      /* next check if it is an array and adjust dimension;
       * note that the expression must resolve during the first
       * pass, otherwise it does not end with defined symbols.
       */
      NextToken();
      if( TestAndSkip(TK_LBRACKET) ) {
        dim = EvalExpr( NULL );
        MustSkip(TK_RBRACKET);
      }
      else {
        dim = 1;
      }
      
      /* now allocate the variable its offset address (will be
       * relocated at the end of pass 1). Variables are allocated
       * by size specifier pool.
       */
      sym->value = curobj->varOffsets[s];
      curobj->varOffsets[s] += (dim<<s);
    } while( TestAndSkip(TK_COMMA) );
    MustSkip(TK_EOL);
  }
}

/* Pass 2 is simple: all the work has been done in the first
 * pass, hence on the second pass we simply skip the block.
 */
void ParseVARpass2()
{
  SkipBlock();
} 
